/*
 *      fhist - file history and comparison tools
 *      Copyright (C) 2000, 2002, 2004, 2008, 2010 Peter Miller
 *
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see
 *      <http://www.gnu.org/licenses/>.
 */

#include <common/ac/stddef.h>
#include <common/ac/stdio.h>
#include <common/ac/sys/stat.h>
#include <libexplain/fflush.h>

#include <common/fcheck.h>
#include <common/output/private.h>
#include <common/output/stdout.h>
#include <common/str.h>
#include <common/error_intl.h>


typedef struct output_stdout_ty output_stdout_ty;
struct output_stdout_ty
{
        output_ty       inherited;
};


static const char *
standard_output(void)
{
        const char      *name;

        name = "standard output";
        return name;
}


static void
destructor(output_ty *this)
{
        explain_fflush_or_die(stdout);
}


static const char *
filename(output_ty *this)
{
        return standard_output();
}


static long
otell(output_ty *this)
{
        return ftell(stdout);
}


static void
oputc(output_ty *fp, int c)
{
        output_stdout_ty *this;

        this = (output_stdout_ty *)fp;
        if (putchar(c) == EOF)
        {
                sub_context_ty  *scp;

                scp = sub_context_new();
                sub_errno_set(scp);
                sub_var_set_charstar(scp, "File_Name", standard_output());
                fatal_intl(scp, i18n("write \"$filename\": $errno"));
                /* NOTREACHED */
        }
}


static void
owrite(output_ty *fp, const void *data, size_t len)
{
        output_stdout_ty *this;

        this = (output_stdout_ty *)fp;
        if (fwrite(data, 1, len, stdout) == EOF)
        {
                sub_context_ty  *scp;

                scp = sub_context_new();
                sub_errno_set(scp);
                sub_var_set_charstar(scp, "File_Name", standard_output());
                fatal_intl(scp, i18n("write \"$filename\": $errno"));
                /* NOTREACHED */
        }
}


static void
oflush(output_ty *this)
{
        if (fflush(stdout))
        {
                sub_context_ty  *scp;

                scp = sub_context_new();
                sub_errno_set(scp);
                sub_var_set_charstar(scp, "File_Name", standard_output());
                fatal_intl(scp, i18n("write \"$filename\": $errno"));
                /* NOTREACHED */
        }
}


static output_vtbl_ty vtbl =
{
        sizeof(output_stdout_ty),
        destructor,
        filename,
        otell,
        oputc,
        output_generic_fputs,
        owrite,
        oflush,
        "stdout",
};


output_ty *
output_stdout(void)
{
        output_ty       *result;
        output_stdout_ty *this;

        result = output_new(&vtbl);
        this = (output_stdout_ty *)result;
        return result;
}
